bl_info = {
    "name": "Cad<Measure",
    "author": "Mr.Nobody",
    "version": (0, 0, 1),
    "blender": (4, 4, 1),
    "location": "3D Viewport > Sidebar > Cad<Measure",
    "description": "Cad like measuring tool for blender",
    "category": "Development",
}

import bpy
import os
import math

def import_node_group(blend_path, group_node_name, link=True):
    existing_node_tree = bpy.data.node_groups.get(group_node_name)
    if existing_node_tree:
        return existing_node_tree

    if not os.path.exists(blend_path):
        print(f"Blend file not found: {blend_path}")
        return None

    if link:
        with bpy.data.libraries.load(blend_path, link=True) as (data_from, data_to):
            if group_node_name in data_from.node_groups:
                data_to.node_groups.append(group_node_name)

    ng = bpy.data.node_groups.get(group_node_name)
    if ng:
        ng.use_fake_user = True
    else:
        print(f"❌ Error: Node tree '{group_node_name}' not found after linking.")
    return ng

def add_cad_measure_obj(size, gn_asset_name):
    bpy.ops.mesh.primitive_plane_add(size=size)
    obj = bpy.context.active_object
    bpy.ops.object.mode_set(mode='EDIT')
    bpy.ops.mesh.select_all(action='SELECT')
    bpy.ops.mesh.merge(type='CENTER')
    bpy.ops.object.mode_set(mode='OBJECT')
    bpy.context.scene.tool_settings.use_snap = True
    bpy.context.scene.tool_settings.snap_elements_base = {'EDGE'}

    mod = obj.modifiers.new(name="Measuring tool", type="NODES")
    mod.node_group = bpy.data.node_groups.get(gn_asset_name)
    return obj

class MESH_OT_add_cad_measure(bpy.types.Operator):
    bl_idname = "mesh.add_cad_measure"
    bl_label = "Add CAD Measure Object"
    bl_options = {"REGISTER", "UNDO"}

    cadify: bpy.props.BoolProperty(default=False)
    axis: bpy.props.EnumProperty(
        name="Axis",
        items=[
            ('X', "X", "Measure on X axis"),
            ('Y', "Y", "Measure on Y axis"),
            ('Z', "Z", "Measure on Z axis"),
        ],
        default='Y'
    )

    @classmethod
    def description(cls, context, properties):
        axis = properties.axis
        return f"Measure on {axis} axis"

    def execute(self, context):
        gn_path = os.path.join(os.path.dirname(__file__), "Node", "CAD.blend")
        gn_asset_name = "CAD"
        import_node_group(gn_path, gn_asset_name, True)

        in_edit_mode = (context.mode == "EDIT_MESH")
        obj = None

        if in_edit_mode:
            obj = context.edit_object
            bpy.ops.object.mode_set(mode="OBJECT")
        else:
            obj = add_cad_measure_obj(4.0, gn_asset_name)
            obj.name = "CAD_Measure"

        obj.rotation_euler = (0.0, 0.0, 0.0)
        if self.axis == 'X':
            obj.rotation_euler[2] = math.radians(270)
        elif self.axis == 'Z':
            obj.rotation_euler[0] = math.radians(90)

        if in_edit_mode:
            bpy.ops.object.mode_set(mode="EDIT")

        if not in_edit_mode and not self.cadify:
            bpy.ops.object.mode_set(mode="EDIT")

        if not self.cadify:
            bpy.ops.object.mode_set(mode="EDIT")
        elif context.mode != 'EDIT_MESH':
            bpy.ops.object.mode_set(mode='EDIT')

        return {"FINISHED"}


class VIEW3D_OT_cadify_viewport(bpy.types.Operator):
    bl_idname = "view3d.cadify_viewport"
    bl_label = "CADIFY Viewport"
    bl_description = "Apply CAD-like viewport settings"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        for area in bpy.context.screen.areas:
            if area.type == "VIEW_3D":
                for space in area.spaces:
                    if space.type == "VIEW_3D":
                        space.shading.light = "FLAT"
                        space.shading.background_type = "VIEWPORT"
                        space.shading.background_color = (1, 1, 1)
                        space.shading.color_type = 'MATERIAL'

        world = bpy.data.worlds.get("World")
        if not world:
            world = bpy.data.worlds.new("World")
        bpy.context.scene.world = world
        world.use_nodes = False
        world.color = (1, 1, 1)

        return {'FINISHED'}


class VIEW3D_PT_my_custom_panel(bpy.types.Panel):
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = "Cad<Measure"
    bl_label = "Cad<Measure"

    def draw(self, context):
        layout = self.layout
        layout.operator("view3d.cadify_viewport", text="CADIFY")
        layout.separator()
        layout.label(text="Add CAD Measure:")
        row = layout.row(align=True)
        row.operator("mesh.add_cad_measure", text="X").axis = 'X'
        row.operator("mesh.add_cad_measure", text="Y").axis = 'Y'
        row.operator("mesh.add_cad_measure", text="Z").axis = 'Z'


def register():
    bpy.utils.register_class(VIEW3D_PT_my_custom_panel)
    bpy.utils.register_class(MESH_OT_add_cad_measure)
    bpy.utils.register_class(VIEW3D_OT_cadify_viewport)

def unregister():
    bpy.utils.unregister_class(VIEW3D_PT_my_custom_panel)
    bpy.utils.unregister_class(MESH_OT_add_cad_measure)
    bpy.utils.unregister_class(VIEW3D_OT_cadify_viewport)

if __name__ == "__main__":
    register()
